import java.util.Arrays;

public class SortTest {
  
  enum TestMode {
    INTEGERS_1000,
    CITIES_50,
    COUNTIES_3000,
  }
  
  static Measurement[] measurements;
  static Sorter[] sorters;
  static Comparable[] data;
  
  public static void main(String[] args) {
    // test1();
    test2();
    // test3();
  }
  
  static void test1() {
    Sorter heapSort = new HeapSort();
    Integer[] values = new Integer[] { 10, 3, 1, 7, 6, 5, 4, 2, 9, 8 };
    heapSort.sort(values);
    for (int x: values) {
      Out.print(" " + x);
    }
  }

  static void test2() {
    Out.println();
    evaluate(TestMode.COUNTIES_3000);
  }
  
  static void test3() {
    Comparable[] values = generateIntegers(20, Order.RANDOM);
    for (Comparable x: values) {
      Out.print(" " + x);
    }
  }
  
  static void initMeasurements() {
    measurements = new Measurement[sorters.length];
    for (int i = 0; i < sorters.length; i++) {
      measurements[i] = new Measurement(sorters[i]);
    }
  }
  
  static void evaluate(TestMode mode) {
    switch(mode) {
      case INTEGERS_1000:
        data = generateIntegers(1000, Order.RANDOM);
        break;
      case CITIES_50:
        data = readData("Cities.txt");
        break;
      case COUNTIES_3000:
        data = readData("Counties.txt");
        break;
    }

    sorters = new Sorter[] { new BubbleSort(), new SelectionSort(),
       new InsertionSort(), new MergeSort(),
       new JavaSort(), new QuickSort(),
       new HeapSort() };

    initMeasurements();
    runMeasurements();
    
    new ExampleUI(measurements, data).show();
  }
  
  private static Comparable[] readData(String filename) {
    In.open(filename);
    int cnt = In.readInt(); In.readLine();
    String[] data = new String[cnt];
    int i = 0;
    while (In.done() && i < cnt) {
      String s = In.readLine();
      // remove CRLF
      data[i] = s.substring(0, s.length()-1);
      i++;
    }
    assert(cnt == i);
    In.close();
    return data;
  }

  public static enum Order { ASCENDING, DESCENDING, RANDOM };
  
  private static Comparable[] generateIntegers(int cnt, Order order) {
    Integer[] arr = new Integer[cnt];
    for (int i = 0; i < cnt; i++) {
      switch (order) {
        case ASCENDING:
          arr[i] = i;
          break;
        case DESCENDING:
          arr[i] = cnt - i;
          break;
        case RANDOM:
          arr[i] = (int) (Math.random() * cnt);
          break;
      }
    }
    return arr;
  }

  static void runMeasurements() {
    final int REPEAT = 50;
    for (Measurement m: measurements) {
      m.timer = Timer.start();
      for (int i = 0; i < REPEAT; i++) {
        m.timer.pause();
        m.sortedData = data.clone();
        m.timer.continu();
        m.sorter.sort(m.sortedData);
      }
      m.timer.stop();
    }
    Arrays.sort(measurements);
  }

}